
package com.adakoda.android.sutameanimationsample;

import android.animation.Animator;
import android.animation.AnimatorListenerAdapter;
import android.animation.AnimatorSet;
import android.animation.ObjectAnimator;
import android.app.Activity;
import android.os.Bundle;
import android.view.View;
import android.view.animation.CycleInterpolator;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

//
// 「ビューオブジェクトをアニメーションさせる」サンプル
//
// 開発環境
// - Eclipse 3.7.2
// - Android SDK Tools Rev.20.0.1 (Android 3.0 (API Level 11) / Android 4.0 (API Level 14))
// - Android SDK Platform-tools Rev.13
// - ADT 20.0.2
//
// 動作確認済デバイス
// - Galaxy Nexus Android 4.0
// ※Galaxy Nexus（画面サイズが1280x720の環境）に特化してレイアウトを作成しているため、
// その他の環境では各画像のサイズが小さく表示される可能性があります
//
public class SutameAnimationSampleActivity extends Activity {

    // グループ1
    private ImageView mView1; // 1.イメージビュー（上から下へ移動）
    private ImageView mView2; // 2.イメージビュー（下から上へ移動）

    // グループ2
    private TextView mView3; // 3.テキストビュー（フェードイン）
    private Button mView4; // 4.ボタン（左から右へ移動）
    private Button mView5; // 5.ボタン（右から左へ移動）

    // グループ3
    private ImageView mView6; // 6.イメージビュー（反時計回りに90度回転）
    private ImageView mView7; // 7.イメージビュー（時計回りに90度回転）

    // 全体をまとめるグループ（グループ1、2、3のセット）
    private AnimatorSet mAnimatorSet;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);

        mView1 = (ImageView) findViewById(R.id.view1);
        mView2 = (ImageView) findViewById(R.id.view2);
        mView3 = (TextView) findViewById(R.id.view3);
        mView4 = (Button) findViewById(R.id.view4);
        mView5 = (Button) findViewById(R.id.view5);
        mView6 = (ImageView) findViewById(R.id.view6);
        mView7 = (ImageView) findViewById(R.id.view7);

        //
        // おまけ
        //

        // タップすると揺れるアニメーション
        mView6.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                ObjectAnimator animator = ObjectAnimator.ofFloat(
                        mView6, "rotation", 0f, 10f);
                animator.setInterpolator(new CycleInterpolator(2));
                animator.setDuration(1500);
                animator.start();
            }
        });
        mView7.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                ObjectAnimator animator = ObjectAnimator.ofFloat(
                        mView7, "rotation", 0f, -10f);
                animator.setInterpolator(new CycleInterpolator(2));
                animator.setDuration(1500);
                animator.start();
            }
        });
    }

    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);
        // アニメーションをキャンセルする
        cancelAnimation();

        // フォーカス時
        if (hasFocus) {
            // アニメーションを開始する
            startAnimation();
        }
    }

    // アニメーションを再生する
    private void startAnimation() {
        if (mAnimatorSet == null) {
            // グループ1のアニメーションを作成する
            AnimatorSet animatorSet1 = makeGroup1Animation();
            // グループ2のアニメーションを作成する
            AnimatorSet animatorSet2 = makeGroup2Animation();
            // グループ3のアニメーションを作成する
            AnimatorSet animatorSet3 = makeGroup3Animation();
            // 全体をまとめるグループのアニメーションを作成する
            mAnimatorSet = makeAllGroupAnimation(animatorSet1, animatorSet2, animatorSet3);
        }
        // アニメーションを再生する
        mAnimatorSet.start();
    }

    // アニメーションをキャンセルする
    private void cancelAnimation() {
        // アニメーションをキャンセルする
        if (mAnimatorSet != null) {
            mAnimatorSet.cancel();
        }
    }

    // グループ1のアニメーションを作成する
    private AnimatorSet makeGroup1Animation() {
        // アニメーション1：上から下移動へ移動（画面外から画面内上部へ）
        ObjectAnimator animator1 = ObjectAnimator.ofFloat(
                mView1, "y", -mView1.getHeight(), mView1.getTop());

        // アニメーション2：下から上移動へ移動（画面外から画面内下部へ）
        ObjectAnimator animator2 = ObjectAnimator.ofFloat(
                mView2, "y", mView2.getBottom(), mView2.getTop());

        // グループ1 （アニメーション1、2のセット）
        AnimatorSet animatorSet1 = new AnimatorSet();
        // アニメーション1、2を同時に再生するように設定
        animatorSet1.playTogether(animator1, animator2);
        animatorSet1.addListener(new AnimatorListenerAdapter() {
            public void onAnimationStart(Animator animator) {
                // アニメーション開始時に表示状態に設定する
                mView1.setVisibility(View.VISIBLE);
                mView2.setVisibility(View.VISIBLE);
            }
        });
        return animatorSet1;
    }

    // グループ2のアニメーションを作成する
    private AnimatorSet makeGroup2Animation() {
        // アニメーション3： フェードイン
        ObjectAnimator animator3 = ObjectAnimator.ofFloat(
                mView3, "alpha", 0f, 1f);

        // アニメーション4：左から右移動へ移動（画面外から画面内へ）
        ObjectAnimator animator4 = ObjectAnimator.ofFloat(
                mView4, "x", mView4.getLeft() - mView4.getWidth(), mView4.getLeft());

        // アニメーション5：右から左移動へ移動（画面から画面内へ）
        ObjectAnimator animator5 = ObjectAnimator.ofFloat(
                mView5, "x", mView5.getLeft() + mView5.getWidth(), mView5.getLeft());

        // グループ2 （アニメーション3、4、5のセット）
        AnimatorSet animatorSet2 = new AnimatorSet();
        // アニメーション3、4、5を同時に再生するように設定
        animatorSet2.playTogether(animator3, animator4, animator5);

        // リスナーを設定
        animatorSet2.addListener(new AnimatorListenerAdapter() {
            public void onAnimationStart(Animator animator) {
                // アニメーション開始時に表示状態に設定する
                mView3.setVisibility(View.VISIBLE);
                mView4.setVisibility(View.VISIBLE);
                mView5.setVisibility(View.VISIBLE);
            }
        });
        return animatorSet2;
    }

    // グループ3のアニメーションを作成する
    private AnimatorSet makeGroup3Animation() {
        // アニメーション6：反時計回りに90度回転（90度から0度へ）
        // 回転の中心を左下設定
        mView6.setPivotX(0);
        mView6.setPivotY(mView6.getHeight());
        ObjectAnimator animator6 = ObjectAnimator.ofFloat(
                mView6, "rotation", 90, 0);

        // アニメーション7：時計回りに90度回転（-90度から0度へ）
        // 回転の中心を右下に設定
        mView7.setPivotX(mView7.getWidth());
        mView7.setPivotY(mView7.getHeight());
        ObjectAnimator animator7 = ObjectAnimator.ofFloat(
                mView7, "rotation", -90, 0);

        // グループ3 （アニメーション6、7のセット）
        AnimatorSet animatorSet3 = new AnimatorSet();
        // アニメーション6、7を同時に再生するように設定
        animatorSet3.playTogether(animator6, animator7);

        // リスナーを設定
        animatorSet3.addListener(new AnimatorListenerAdapter() {
            public void onAnimationStart(Animator animator) {
                // アニメーション開始時に表示状態に設定する
                mView6.setVisibility(View.VISIBLE);
                mView7.setVisibility(View.VISIBLE);
            }
        });
        return animatorSet3;
    }

    // 全体をまとめるグループのアニメーションを作成する
    private AnimatorSet makeAllGroupAnimation(
            AnimatorSet animatorSet1, AnimatorSet animatorSet2, AnimatorSet animatorSet3) {
        // 全体をまとめるグループ（グループ1、2、3のセット）
        AnimatorSet animatorSet = new AnimatorSet();
        // グループ1、2、3を順番に再生するように設定
        animatorSet.playSequentially(animatorSet1, animatorSet2, animatorSet3);
        // アニメーションの開始遅延時間を設定
        animatorSet.setStartDelay(100);
        // すべてのアニメーションの継続時間を同じ値に設定
        animatorSet.setDuration(500);

        // リスナーを設定
        animatorSet.addListener(new AnimatorListenerAdapter() {
            @Override
            public void onAnimationStart(Animator animation) {
                super.onAnimationStart(animation);
                // アニメーション開始時、すべてのビューを非表示にする
                mView1.setVisibility(View.INVISIBLE);
                mView2.setVisibility(View.INVISIBLE);
                mView3.setVisibility(View.INVISIBLE);
                mView4.setVisibility(View.INVISIBLE);
                mView5.setVisibility(View.INVISIBLE);
                mView6.setVisibility(View.INVISIBLE);
                mView7.setVisibility(View.INVISIBLE);
            }
        });
        return animatorSet;
    }

}
